import "./Autocomplete.css";
import { Keys, augmentElement, forwardRef, isNilOrEmpty, mergeProps, omitProps, useCommittedRef, useControllableState, useEventCallback, useId, useRefState } from "../../shared";
import { useCallback, useRef, useState } from "react";
import { HiddenAutocomplete } from "./HiddenAutocomplete";
import { Listbox, OptionKeyProp } from "../../listbox";
import { Overlay, isDevToolsBlurEvent, isTargetParent, useFocusWithin, usePopup, useTriggerWidth } from "../../overlay";
import { SearchInput } from "../../text-input";
import { useFieldInputProps } from "../../field";
import { getItemText, useCollectionSearch, useOnlyCollectionItems } from "../../collection";
import { isNil } from "lodash";
import { useDebouncedCallback } from "./useDebouncedCallback";
import { useDeferredValue } from "./useDeferredValue"; // Used to generate OverlayProps instead of any in the auto-generated documentation
// eslint-disable-next-line @typescript-eslint/no-empty-interface

import { jsx as _jsx } from "react/jsx-runtime";
import { jsxs as _jsxs } from "react/jsx-runtime";
export function InnerAutocomplete(props) {
  var _ref;

  const [fieldProps] = useFieldInputProps();
  const {
    id,
    open: openProp,
    defaultOpen,
    value: valueProp,
    defaultValue,
    placeholder,
    onSearch,
    loading,
    clearOnSelect,
    noResultsMessage,
    minCharacters = 1,
    required,
    validationState,
    onChange,
    onOpenChange,
    icon,
    direction = "bottom",
    align = "start",
    autoFocus,
    name,
    fluid,
    disabled,
    allowFlip = true,
    allowPreventOverflow = true,
    zIndex = 10000,
    active,
    focus,
    hover,
    "aria-label": ariaLabel,
    // Usually provided by the field inputs.
    "aria-labelledby": ariaLabelledBy,
    "aria-describedby": ariaDescribedBy,
    overlayProps: {
      id: menuId,
      style: {
        width: menuWidth,
        ...menuStyle
      } = {},
      ...menuProps
    } = {},
    as = "input",
    children,
    forwardedRef,
    ...rest
  } = mergeProps(props, omitProps(fieldProps, ["size"]));
  const [focusedItem, setFocusedItem] = useState(null);
  const [queryRef, setQuery] = useRefState("");
  const [value, setValue] = useControllableState(valueProp, defaultValue, null, {
    onChange: useCallback((newValue, {
      isInitial,
      isControlled
    }) => {
      // Keep query in sync with the initial or controlled value.
      if (isInitial || isControlled) {
        setQuery(newValue !== null && newValue !== void 0 ? newValue : "");
      }

      return undefined;
    }, [setQuery])
  });
  const {
    isOpen,
    setIsOpen,
    triggerElement,
    overlayElement,
    triggerProps,
    overlayProps
  } = usePopup("listbox", {
    id: menuId,
    open: openProp,
    defaultOpen,
    onOpenChange,
    hideOnEscape: true,
    hideOnLeave: true,
    restoreFocus: true,
    autoFocus,
    // An autocomplete take care of his own trigger logic.
    trigger: null,
    position: `${direction}-${align}`,
    offset: [0, 4],
    allowFlip,
    allowPreventOverflow
  });
  const listboxRef = useRef();
  const triggerRef = useCommittedRef(triggerElement);
  const [results, searchCollection] = useCollectionSearch(children, {
    onSearch
  }); // Required to support selection when there are sections.

  const items = useOnlyCollectionItems(results);
  const open = useCallback(event => {
    setIsOpen(event, true);
  }, [setIsOpen]);
  const close = useCallback(event => {
    setIsOpen(event, false);
    setFocusedItem(null);
  }, [setIsOpen, setFocusedItem]);
  const setSelection = useCallback((event, newKey) => {
    var _newValue;

    let newValue = null;

    if (!isNil(newKey)) {
      const selectedItem = items.find(x => x.key === newKey);

      if (!isNil(selectedItem)) {
        newValue = getItemText(selectedItem);
      }
    }

    if (value !== newValue) {
      if (!isNil(onChange)) {
        onChange(event, isNil(newKey) ? null : {
          key: newKey,
          value: newValue
        });
      }

      setValue(newValue);
    }

    setQuery(clearOnSelect ? "" : (_newValue = newValue) !== null && _newValue !== void 0 ? _newValue : "", true);
  }, [items, onChange, clearOnSelect, value, setValue, setQuery]);
  const clear = useCallback(event => {
    setSelection(event, null);
  }, [setSelection]);
  const reset = useCallback(() => {
    // Reset the value to the last selected one.
    if (value !== queryRef.current) {
      setQuery(value !== null && value !== void 0 ? value : "", true);
    }
  }, [value, queryRef, setQuery]);
  const search = useDebouncedCallback((event, query) => {
    if (query.trim().length >= minCharacters) {
      searchCollection(event, query);
      open(event);
    } else if (isNilOrEmpty(query)) {
      clear(event);
      close(event);
    } else {
      close(event);
    }
  }, 200);
  const selectItem = useCallback((event, key) => {
    setSelection(event, key);
    close(event);
  }, [setSelection, close]);
  const triggerWidth = useTriggerWidth(triggerElement);
  const triggerFocusWithinProps = useFocusWithin({
    onBlur: useEventCallback(event => {
      if (!isDevToolsBlurEvent(triggerRef)) {
        // Close the menu when the focus switch from the trigger to somewhere else than the menu or the trigger.
        if (!isTargetParent(event.relatedTarget, triggerElement) && !isTargetParent(event.relatedTarget, overlayElement)) {
          close(event);
          reset();
        }
      }
    })
  });
  const handleTriggerKeyDown = useEventCallback(event => {
    switch (event.key) {
      case Keys.arrowDown:
        if (isOpen) {
          var _listboxRef$current;

          event.preventDefault();
          const activeElement = (_listboxRef$current = listboxRef.current) === null || _listboxRef$current === void 0 ? void 0 : _listboxRef$current.focusManager.focusNext();
          setFocusedItem({
            id: activeElement.id,
            key: activeElement.getAttribute(OptionKeyProp)
          });
        }

        break;

      case Keys.arrowUp:
        if (isOpen) {
          var _listboxRef$current2;

          event.preventDefault();
          const activeElement = (_listboxRef$current2 = listboxRef.current) === null || _listboxRef$current2 === void 0 ? void 0 : _listboxRef$current2.focusManager.focusPrevious();
          setFocusedItem({
            id: activeElement.id,
            key: activeElement.getAttribute(OptionKeyProp)
          });
        }

        break;

      case Keys.home:
        if (isOpen) {
          var _listboxRef$current3;

          event.preventDefault();
          const activeElement = (_listboxRef$current3 = listboxRef.current) === null || _listboxRef$current3 === void 0 ? void 0 : _listboxRef$current3.focusManager.focusFirst();
          setFocusedItem({
            id: activeElement.id,
            key: activeElement.getAttribute(OptionKeyProp)
          });
        }

        break;

      case Keys.end:
        if (isOpen) {
          var _listboxRef$current4;

          event.preventDefault();
          const activeElement = (_listboxRef$current4 = listboxRef.current) === null || _listboxRef$current4 === void 0 ? void 0 : _listboxRef$current4.focusManager.focusLast();
          setFocusedItem({
            id: activeElement.id,
            key: activeElement.getAttribute(OptionKeyProp)
          });
        }

        break;

      case Keys.esc:
        if (isOpen) {
          // Do not remove otherwise the SearchInput will clear the input on esc.
          event.stopPropagation();
          event.preventDefault();
          close(event);
        }

        break;

      case Keys.enter:
        if (isOpen) {
          event.preventDefault();

          if (!isNil(focusedItem)) {
            selectItem(event, focusedItem.key);
          }
        }

        break;
    }
  });
  const handleTriggerChange = useEventCallback((event, query) => {
    setQuery(query, true);
    search(event, query);
  });
  const handleListboxSelectionChange = useEventCallback((event, newKeys) => {
    var _newKeys$;

    selectItem(event, (_newKeys$ = newKeys[0]) !== null && _newKeys$ !== void 0 ? _newKeys$ : null);
  });
  const handleListboxFocusChange = useEventCallback((_event, newKey, activeElement) => {
    setFocusedItem({
      id: activeElement.id,
      key: newKey
    });
  });
  const triggerId = useId(id, id ? null : "o-ui-autocomplete-trigger");
  const iconMarkup = icon && augmentElement(icon, {
    className: "o-ui-autocomplete-icon",
    size: "sm"
  });

  const listboxMarkup = /*#__PURE__*/_jsx(Listbox, {
    nodes: results // An autocomplete doesn't support any persisted selected keys.
    ,
    selectedKeys: [],
    onSelectionChange: handleListboxSelectionChange,
    onFocusChange: handleListboxFocusChange,
    focusOnHover: true,
    useVirtualFocus: true,
    tabbable: false,
    fluid: true,
    className: "o-ui-autocomplete-listbox",
    "aria-label": ariaLabel,
    "aria-labelledby": isNil(ariaLabel) ? ariaLabelledBy !== null && ariaLabelledBy !== void 0 ? ariaLabelledBy : triggerId : undefined,
    "aria-describedby": ariaDescribedBy,
    ref: listboxRef
  });

  const noResultsMarkup = /*#__PURE__*/_jsx("div", {
    className: "o-ui-autocomplete-no-results",
    children: noResultsMessage !== null && noResultsMessage !== void 0 ? noResultsMessage : "No results found."
  });

  return /*#__PURE__*/_jsxs("div", {
    children: [/*#__PURE__*/_jsx(HiddenAutocomplete, {
      name: name,
      value: value,
      required: required,
      validationState: validationState,
      disabled: disabled
    }), /*#__PURE__*/_jsx(SearchInput, { ...mergeProps(rest, {
        id: triggerId,
        value: queryRef.current,
        placeholder,
        icon: iconMarkup !== null && iconMarkup !== void 0 ? iconMarkup : null,
        onChange: handleTriggerChange,
        onKeyDown: handleTriggerKeyDown,
        autoFocus,
        loading: useDeferredValue(loading, 100, false),
        disabled,
        validationState,
        fluid,
        active,
        focus,
        hover,
        className: "o-ui-autocomplete-trigger",
        type: "text",
        role: "combobox",
        autoCorrect: "off",
        spellCheck: "false",
        autoComplete: "off",
        "aria-activedescendant": focusedItem === null || focusedItem === void 0 ? void 0 : focusedItem.id,
        "aria-autocomplete": "list",
        "aria-label": ariaLabel,
        "aria-labelledby": isNil(ariaLabel) ? ariaLabelledBy : undefined,
        "aria-describedby": ariaDescribedBy,
        as,
        ref: forwardedRef
      }, triggerProps, triggerFocusWithinProps)
    }), /*#__PURE__*/_jsx(Overlay, { ...mergeProps(menuProps, {
        show: isOpen && (!loading || results.length > 0),
        zIndex,
        className: `o-ui-autocomplete-menu ${results.length > 0 ? "" : "o-ui-autocomplete-menu-no-results"}`,
        style: { ...menuStyle,
          width: (_ref = menuWidth !== null && menuWidth !== void 0 ? menuWidth : triggerWidth) !== null && _ref !== void 0 ? _ref : undefined
        }
      }, overlayProps),
      children: results.length > 0 ? listboxMarkup : noResultsMarkup
    })]
  });
}
export const Autocomplete = forwardRef((props, ref) => /*#__PURE__*/_jsx(InnerAutocomplete, { ...props,
  forwardedRef: ref
}));
Autocomplete.displayName = "Autocomplete";