import "./Listbox.css";
import { Keys, appendEventKey, cssModule, forwardRef, mergeProps, useAutoFocusChild, useControllableState, useDisposables, useEventCallback, useFocusManager, useFocusScope, useId, useKeyedRovingFocus, useMergedRefs, useRefState } from "../../shared";
import { Box } from "../../box";
import { NodeType, useCollection, useOnlyCollectionItems } from "../../collection";
import { useImperativeHandle, useMemo } from "react";
import { ListboxContext } from "./ListboxContext";
import { ListboxOption } from "./ListboxOption";
import { ListboxSection } from "./ListboxSection";
import { isNil, isNumber } from "lodash";
import { jsx as _jsx } from "react/jsx-runtime";
export const OptionKeyProp = "data-o-ui-key";
const SelectionMode = {
  none: "none",
  single: "single",
  multiple: "multiple"
}; // used to generate CollectionNode[] instead of any[] in the auto-generated documentation
// eslint-disable-next-line @typescript-eslint/no-empty-interface

function useCollectionNodes(children, nodes) {
  const collectionNodes = useCollection(children);
  return nodes !== null && nodes !== void 0 ? nodes : collectionNodes;
}

function useSelectionManager(items, {
  selectedKeys
}) {
  return useMemo(() => {
    const toggleKey = key => {
      return selectedKeys.includes(key) ? selectedKeys.filter(x => x !== key) : [...selectedKeys, key];
    };

    const replaceSelection = key => {
      return [key];
    };

    const extendSelection = toKey => {
      if (selectedKeys.length > 0) {
        const lastKey = selectedKeys[selectedKeys.length - 1];
        const newKeys = new Set(selectedKeys);
        let startIndex = items.findIndex(x => x.key === lastKey);
        let endIndex = items.findIndex(x => x.key === toKey); // Support both directions.

        if (startIndex > endIndex) {
          [startIndex, endIndex] = [endIndex, startIndex];
        }

        for (let i = startIndex; i <= endIndex; i += 1) {
          newKeys.add(items[i].key);
        }

        return Array.from(newKeys);
      }

      return selectedKeys;
    };

    return {
      selectedKeys: selectedKeys,
      toggleKey,
      replaceSelection,
      extendSelection
    };
  }, [selectedKeys, items]);
}

export function InnerListbox({
  id,
  selectedKeys: selectedKeysProp,
  defaultSelectedKeys,
  onSelectionChange,
  onFocusChange,
  selectionMode = "single",
  nodes: nodesProp,
  autoFocus,
  // TODO: Could it be removed now that useImperativeHandle expose the focus? If yes, also remove from Menu (which might not event need the useImperativeHandle)
  defaultFocusTarget,
  focusOnHover,
  useVirtualFocus,
  tabbable = true,
  fluid,
  "aria-label": ariaLabel,
  "aria-labelledby": ariaLabelledBy,
  as = "div",
  children,
  forwardedRef,
  ...rest
}) {
  var _ref;

  const [selectedKeys, setSelectedKeys] = useControllableState(selectedKeysProp, defaultSelectedKeys, []);
  const [searchQueryRef, setSearchQuery] = useRefState("");
  const [focusScope, setFocusRef] = useFocusScope();
  const containerRef = useMergedRefs(setFocusRef);
  const nodes = useCollectionNodes(children, nodesProp);
  const items = useOnlyCollectionItems(nodes);
  const selectionManager = useSelectionManager(items, {
    selectedKeys
  });
  const focusManager = useFocusManager(focusScope, {
    isVirtual: useVirtualFocus,
    keyProp: OptionKeyProp
  }); // Would be nice to find a better way to give control over the focused item to the parent.

  useImperativeHandle(forwardedRef, () => {
    const element = containerRef.current;
    element.focusManager = focusManager;
    return element;
  });

  const updateSelectedKeys = (event, newKeys) => {
    if (!isNil(onSelectionChange)) {
      onSelectionChange(event, newKeys);
    }

    if (selectionMode !== SelectionMode.none) {
      setSelectedKeys(newKeys);
    }
  };

  const handleSelectOption = useEventCallback((event, key) => {
    let newKeys;

    if (selectionMode === SelectionMode.multiple) {
      newKeys = selectionManager.toggleKey(key);
    } else {
      newKeys = selectionManager.replaceSelection(key);
    }

    updateSelectedKeys(event, newKeys);
  });
  const handleFocusOption = useEventCallback((event, key, activeElement) => {
    if (!isNil(onFocusChange)) {
      onFocusChange(event, key, activeElement);
    }
  });
  const searchDisposables = useDisposables();
  const handleKeyDown = useEventCallback(event => {
    searchDisposables.dispose();

    switch (event.key) {
      case Keys.arrowDown:
        {
          event.preventDefault();
          const activeElement = focusManager.focusNext();
          const key = activeElement.getAttribute(OptionKeyProp);

          if (!isNil(onFocusChange)) {
            onFocusChange(event, key, activeElement);
          }

          if (selectionMode === SelectionMode.multiple) {
            if (event.shiftKey) {
              const newKeys = selectionManager.toggleKey(key);
              updateSelectedKeys(event, newKeys);
            }
          }

          break;
        }

      case Keys.arrowUp:
        {
          event.preventDefault();
          const activeElement = focusManager.focusPrevious();
          const key = activeElement.getAttribute(OptionKeyProp);

          if (!isNil(onFocusChange)) {
            onFocusChange(event, key, activeElement);
          }

          if (selectionMode === SelectionMode.multiple) {
            if (event.shiftKey) {
              const newKeys = selectionManager.toggleKey(key);
              updateSelectedKeys(event, newKeys);
            }
          }

          break;
        }

      case Keys.home:
        {
          event.preventDefault();
          const activeElement = focusManager.focusFirst();

          if (!isNil(onFocusChange)) {
            onFocusChange(event, activeElement.getAttribute(OptionKeyProp), activeElement);
          }

          break;
        }

      case Keys.end:
        {
          event.preventDefault();
          const activeElement = focusManager.focusLast();

          if (!isNil(onFocusChange)) {
            onFocusChange(event, activeElement.getAttribute(OptionKeyProp), activeElement);
          }

          break;
        }

      case Keys.space:
        event.preventDefault();

        if (selectionMode === SelectionMode.multiple) {
          if (event.shiftKey) {
            const newKeys = selectionManager.extendSelection(document.activeElement.getAttribute(OptionKeyProp));
            updateSelectedKeys(event, newKeys);
          }
        }

        break;
      // eslint-disable-next-line no-fallthrough

      default:
        if (event.key.length === 1) {
          event.preventDefault();
          const query = appendEventKey(searchQueryRef.current, event.key);
          setSearchQuery(query);
          focusManager.search(query); // Clear search query.

          searchDisposables.setTimeout(() => {
            setSearchQuery("");
          }, 350);
        }

    }
  });
  useKeyedRovingFocus(focusScope, selectionManager.selectedKeys[0], {
    keyProp: OptionKeyProp,
    isDisabled: !tabbable
  });
  useAutoFocusChild(focusManager, {
    target: (_ref = selectionMode !== SelectionMode.none ? selectionManager.selectedKeys[0] : undefined) !== null && _ref !== void 0 ? _ref : defaultFocusTarget,
    isDisabled: !autoFocus,
    delay: isNumber(autoFocus) ? autoFocus : undefined
  });
  const rootId = useId(id, id ? null : "o-ui-listbox");

  const renderOption = ({
    key,
    index,
    elementType: As = ListboxOption,
    ref,
    content,
    props = {},
    tooltip
  }) => /*#__PURE__*/_jsx(As, { ...mergeProps(props, {
      id: `${rootId}-option-${index}`,
      key,
      ref,
      item: {
        key: key,
        tooltip
      }
    }),
    children: content
  });

  const renderSection = ({
    key,
    index,
    elementType: As = ListboxSection,
    ref,
    props = {},
    items: sectionItems
  }) => {
    if (sectionItems.length === 0) {
      return null;
    }

    return /*#__PURE__*/_jsx(As, { ...mergeProps(props, {
        id: `${rootId}-section-${index}`,
        key,
        ref
      }),
      children: sectionItems.map(x => renderOption(x))
    });
  };

  return /*#__PURE__*/_jsx(Box, { ...mergeProps(rest, {
      id: rootId,
      className: cssModule("o-ui-listbox", fluid && "fluid"),
      onKeyDown: handleKeyDown,
      role: "listbox",
      "aria-label": ariaLabel,
      "aria-labelledby": isNil(ariaLabel) ? ariaLabelledBy : undefined,
      "aria-multiselectable": selectionMode === SelectionMode.multiple ? true : undefined,
      as,
      ref: containerRef
    }),
    children: /*#__PURE__*/_jsx(ListboxContext.Provider, {
      value: {
        selectedKeys: selectionManager.selectedKeys,
        onSelect: handleSelectOption,
        onFocus: handleFocusOption,
        focusManager,
        focusOnHover
      },
      children: nodes.map(node => {
        switch (node.type) {
          case NodeType.item:
            return renderOption(node);

          case NodeType.section:
            return renderSection(node);

          default:
            return null;
        }
      })
    })
  });
}
export const Listbox = forwardRef((props, ref) => /*#__PURE__*/_jsx(InnerListbox, { ...props,
  forwardedRef: ref
}));
Listbox.displayName = "Listbox";