import "./Menu.css";
import { Keys, appendEventKey, cssModule, forwardRef, mergeProps, useAutoFocusChild, useControllableState, useDisposables, useEventCallback, useFocusManager, useFocusScope, useId, useMergedRefs, useRefState, useRovingFocus } from "../../shared";
import { Box } from "../../box";
import { NodeType, useCollection } from "../../collection";
import { MenuContext } from "./MenuContext";
import { MenuItem } from "./MenuItem";
import { MenuSection } from "./MenuSection";
import { isNil, isNumber } from "lodash";
import { jsx as _jsx } from "react/jsx-runtime";
export const ItemKeyProp = "data-o-ui-key";
export function InnerMenu({
  id,
  selectedKeys: selectedKeysProp,
  defaultSelectedKeys,
  onSelectionChange,
  selectionMode = "none",
  autoFocus,
  defaultFocusTarget,
  fluid,
  "aria-label": ariaLabel,
  "aria-labelledby": ariaLabelledBy,
  as = "ul",
  children,
  forwardedRef,
  ...rest
}) {
  var _ref;

  const [selectedKeys, setSelectedKeys] = useControllableState(selectedKeysProp, defaultSelectedKeys, []);
  const [searchQueryRef, setSearchQuery] = useRefState("");
  const [focusScope, setFocusRef] = useFocusScope();
  const containerRef = useMergedRefs(setFocusRef, forwardedRef);
  const focusManager = useFocusManager(focusScope, {
    keyProp: ItemKeyProp
  });
  const handleSelectItem = useEventCallback((event, key) => {
    let newKeys;

    if (selectionMode === "multiple") {
      newKeys = selectedKeys.includes(key) ? selectedKeys.filter(x => x !== key) : [...selectedKeys, key];
    } else {
      newKeys = selectedKeys.includes(key) ? [] : [key];
    }

    if (!isNil(onSelectionChange)) {
      onSelectionChange(event, newKeys);
    }

    if (selectionMode !== "none") {
      setSelectedKeys(newKeys);
    }
  });
  const searchDisposables = useDisposables();
  const handleKeyDown = useEventCallback(event => {
    searchDisposables.dispose();

    switch (event.key) {
      case Keys.arrowDown:
        {
          event.preventDefault();
          focusManager.focusNext();
          break;
        }

      case Keys.arrowUp:
        {
          event.preventDefault();
          focusManager.focusPrevious();
          break;
        }

      case Keys.home:
        event.preventDefault();
        focusManager.focusFirst();
        break;

      case Keys.end:
        event.preventDefault();
        focusManager.focusLast();
        break;
      // eslint-disable-next-line no-fallthrough

      default:
        if (event.key.length === 1) {
          event.preventDefault();
          const query = appendEventKey(searchQueryRef.current, event.key);
          setSearchQuery(query);
          focusManager.search(query); // Clear search query.

          searchDisposables.setTimeout(() => {
            setSearchQuery("");
          }, 350);
        }

    }
  });
  useRovingFocus(focusScope);
  useAutoFocusChild(focusManager, {
    target: (_ref = selectionMode !== "none" ? selectedKeys[0] : undefined) !== null && _ref !== void 0 ? _ref : defaultFocusTarget,
    isDisabled: !autoFocus,
    delay: isNumber(autoFocus) ? autoFocus : undefined,
    onNotFound: useEventCallback(() => {
      var _containerRef$current;

      // Ensure keyboard navigation is available.
      (_containerRef$current = containerRef.current) === null || _containerRef$current === void 0 ? void 0 : _containerRef$current.focus();
    })
  });
  const nodes = useCollection(children);
  const rootId = useId(id, id ? null : "o-ui-menu");

  const renderOption = ({
    key,
    index,
    elementType: As = MenuItem,
    ref,
    content,
    props,
    tooltip
  }) => /*#__PURE__*/_jsx(As, { ...mergeProps(props, {
      id: `${rootId}-item-${index}`,
      key,
      ref,
      item: {
        key,
        tooltip
      }
    }),
    children: content
  });

  const renderSection = ({
    key,
    index,
    elementType: As = MenuSection,
    ref,
    props,
    items: sectionItems
  }) => {
    if (sectionItems.length === 0) {
      return null;
    }

    return /*#__PURE__*/_jsx(As, { ...mergeProps(props, {
        id: `${rootId}-section-${index}`,
        key,
        ref
      }),
      children: sectionItems.map(x => renderOption(x))
    });
  };

  const renderDivider = ({
    key,
    elementType: As,
    ref,
    content,
    props
  }) => /*#__PURE__*/_jsx(As, { ...mergeProps(props, {
      className: "o-ui-menu-divider",
      as: "li",
      key,
      ref
    }),
    children: content
  });

  return /*#__PURE__*/_jsx(Box, { ...mergeProps(rest, {
      id: rootId,
      className: cssModule("o-ui-menu", fluid && "fluid"),
      onKeyDown: handleKeyDown,
      role: "menu",
      "aria-orientation": "vertical",
      "aria-label": ariaLabel,
      "aria-labelledby": isNil(ariaLabel) ? ariaLabelledBy : undefined,
      as,
      ref: containerRef
    }),
    children: /*#__PURE__*/_jsx(MenuContext.Provider, {
      value: {
        selectedKeys,
        selectionMode,
        onSelect: handleSelectItem
      },
      children: nodes.map(node => {
        switch (node.type) {
          case NodeType.item:
            return renderOption(node);

          case NodeType.section:
            return renderSection(node);

          case NodeType.divider:
            return renderDivider(node);

          default:
            return null;
        }
      })
    })
  });
}
export const Menu = forwardRef((props, ref) => /*#__PURE__*/_jsx(InnerMenu, { ...props,
  forwardedRef: ref
}));
Menu.displayName = "Menu";