import { Children, useCallback } from "react";
import { DisclosureContext } from "../../disclosure";
import { FocusTarget, Keys, augmentElement, forwardRef, mergeProps, resolveChildren, useChainedEventCallback, useEventCallback, useId, useRefState } from "../../shared";
import { MenuTriggerContext } from "./MenuTriggerContext";
import { Overlay, usePopup } from "../../overlay";
import { isNil } from "lodash";
import { jsx as _jsx } from "react/jsx-runtime";
import { jsxs as _jsxs } from "react/jsx-runtime";
export function InnerMenuTrigger({
  id,
  open: openProp,
  defaultOpen,
  onOpenChange,
  closeOnSelect = true,
  direction = "bottom",
  align = "start",
  allowFlip,
  allowPreventOverflow,
  zIndex = 10000,
  as = "div",
  children,
  forwardedRef,
  ...rest
}) {
  var _trigger$props$ariaL;

  const [focusTargetRef, setFocusTarget] = useRefState(null);
  const handleOpenChange = useChainedEventCallback(onOpenChange, (_event, isVisible) => {
    if (isVisible) {
      // Focusing the first item on open if nore are already set to be focused.
      if (isNil(focusTargetRef.current)) {
        setFocusTarget(FocusTarget.first);
      }
    }
  });
  const {
    isOpen,
    setIsOpen,
    triggerProps,
    overlayProps
  } = usePopup("menu", {
    id,
    open: openProp,
    defaultOpen,
    onOpenChange: handleOpenChange,
    hideOnEscape: true,
    hideOnLeave: true,
    hideOnOutsideClick: true,
    autoFocus: false,
    restoreFocus: true,
    position: `${direction}-${align}`,
    offset: [0, 4],
    allowFlip,
    allowPreventOverflow
  });
  const [trigger, menu] = Children.toArray(resolveChildren(children));

  if (isNil(trigger) || isNil(menu)) {
    throw new Error("A menu trigger must have exactly 2 children.");
  }

  const open = useCallback((event, focusTarget) => {
    setFocusTarget(focusTarget);
    setIsOpen(event, true);
  }, [setIsOpen, setFocusTarget]);
  const close = useCallback(event => {
    setFocusTarget(null);
    setIsOpen(event, false);
  }, [setIsOpen, setFocusTarget]); // Open the menu on up & down arrow keydown.

  const handleTriggerKeyDown = useEventCallback(event => {
    switch (event.key) {
      case Keys.arrowDown:
        event.preventDefault();
        open(event, FocusTarget.first);
        break;

      case Keys.arrowUp:
        event.preventDefault();
        open(event, FocusTarget.last);
        break;
    }
  });
  const handleSelectionChange = useEventCallback(event => {
    if (closeOnSelect) {
      close(event);
    }
  });
  const triggerId = useId(trigger.props.id, trigger.props.id ? null : "o-ui-menu-trigger");
  const triggerMarkup = augmentElement(trigger, mergeProps({
    id: triggerId,
    onKeyDown: !isOpen ? handleTriggerKeyDown : undefined
  }, triggerProps));
  const menuMarkup = augmentElement(menu, {
    onSelectionChange: handleSelectionChange,
    // Must be conditional to isOpen otherwise it will steal the focus from the trigger when selecting
    // a value because the menu re-render before the exit animation is done.
    autoFocus: isOpen,
    defaultFocusTarget: focusTargetRef.current,
    "aria-labelledby": (_trigger$props$ariaL = trigger.props["aria-labelledby"]) !== null && _trigger$props$ariaL !== void 0 ? _trigger$props$ariaL : triggerId,
    "aria-describedby": trigger.props["aria-describedby"]
  });
  return /*#__PURE__*/_jsxs(MenuTriggerContext.Provider, {
    value: {
      isOpen,
      open,
      close
    },
    children: [/*#__PURE__*/_jsx(DisclosureContext.Provider, {
      value: {
        isOpen
      },
      children: triggerMarkup
    }), /*#__PURE__*/_jsx(Overlay, { ...mergeProps(rest, {
        zIndex,
        as,
        ref: forwardedRef
      }, overlayProps),
      children: menuMarkup
    })]
  });
}
export const MenuTrigger = forwardRef((props, ref) => /*#__PURE__*/_jsx(InnerMenuTrigger, { ...props,
  forwardedRef: ref
}));
MenuTrigger.displayName = "MenuTrigger";