import "./NumberInput.css";
import { Box } from "../../box";
import { CarretIcon } from "../../icons";
import { useCallback } from "react";
import { cssModule, forwardRef, mergeProps, omitProps, useControllableState, useEventCallback } from "../../shared";
import { isNil } from "lodash";
import { useFieldInputProps } from "../../field";
import { useInput, useInputIcon, wrappedInputPropsAdapter } from "../../input";
import { useMemo } from "react";
import { useToolbarProps } from "../../toolbar"; // used to generate BoxProps instead of any in the auto-generated documentation
// eslint-disable-next-line @typescript-eslint/no-empty-interface

import { jsx as _jsx } from "react/jsx-runtime";
import { jsxs as _jsxs } from "react/jsx-runtime";
import { Fragment as _Fragment } from "react/jsx-runtime";

function Spinner({
  onIncrement,
  onDecrement,
  onFocus,
  disabled,
  ...rest
}) {
  const handleIncrement = useEventCallback(event => {
    onIncrement(event);
  });
  const handleDecrement = useEventCallback(event => {
    onDecrement(event);
  });
  return /*#__PURE__*/_jsxs("div", { ...mergeProps(rest, {
      className: "o-ui-number-input-spinner",
      "aria-hidden": true
    }),
    children: [/*#__PURE__*/_jsx("button", {
      onClick: handleIncrement,
      className: "o-ui-number-input-spinner-increment",
      type: "button",
      tabIndex: -1,
      disabled: disabled,
      onFocus: onFocus,
      children: /*#__PURE__*/_jsx(CarretIcon, {
        size: "xs"
      })
    }), /*#__PURE__*/_jsx("button", {
      onClick: handleDecrement,
      className: "o-ui-number-input-spinner-decrement",
      type: "button",
      tabIndex: -1,
      disabled: disabled,
      onFocus: onFocus,
      children: /*#__PURE__*/_jsx(CarretIcon, {
        size: "xs",
        className: "o-ui-rotate-180"
      })
    })]
  });
}

function countDecimalPlaces(value) {
  return (value.toString().split(".")[1] || []).length;
}

function toNumber(value) {
  const result = parseFloat(value);

  if (isNaN(result)) {
    return null;
  }

  return result;
}

function toFixed(value, precision) {
  return parseFloat(value.toFixed(precision));
}

export function InnerNumberInput(props) {
  const [toolbarProps] = useToolbarProps();
  const [fieldProps] = useFieldInputProps();
  const {
    id,
    value,
    defaultValue,
    placeholder,
    min,
    max,
    step = 1,
    clampValue = true,
    required,
    validationState,
    onChange,
    autoFocus,
    icon,
    disabled,
    readOnly,
    fluid,
    loading,
    active,
    focus,
    hover,
    wrapperProps: wrapperPropsProp,
    as = "div",
    forwardedRef,
    ...rest
  } = mergeProps(props, omitProps(toolbarProps, ["orientation"]), omitProps(wrappedInputPropsAdapter(fieldProps), ["size"]));
  const [inputValue, setValue] = useControllableState(value, defaultValue, null);

  const updateValue = (event, newValue) => {
    setValue(newValue);

    if (!isNil(onChange)) {
      onChange(event, newValue);
    }
  };

  const validateRange = useCallback(newValue => {
    let isAboveMax = false;
    let isBelowMin = false;

    if (!isNil(newValue)) {
      if (!isNil(max) && newValue > max) {
        isAboveMax = true;
      } else if (!isNil(min) && newValue < min) {
        isBelowMin = true;
      }
    }

    return {
      isInRange: !isAboveMax && !isBelowMin,
      isAboveMax,
      isBelowMin
    };
  }, [min, max]);
  const isInRange = useMemo(() => validateRange(inputValue).isInRange, [inputValue, validateRange]);

  const clamp = event => {
    const {
      isAboveMax,
      isBelowMin
    } = validateRange(inputValue);

    if (isBelowMin) {
      updateValue(event, min);
    } else if (isAboveMax) {
      updateValue(event, max);
    }
  };

  const applyStep = (event, factor) => {
    if (!isNil(inputValue)) {
      const precision = Math.max(countDecimalPlaces(inputValue), countDecimalPlaces(step));
      const newValue = toFixed(inputValue + factor * step, precision);
      const {
        isInRange: inRange
      } = validateRange(newValue);

      if (inRange) {
        updateValue(event, newValue);
      } else {
        if (clampValue) {
          clamp(event);
        }
      }
    } else {
      updateValue(event, factor * step);
    }
  };

  const handleChange = useEventCallback(event => {
    const newValue = toNumber(event.target.value);
    updateValue(event, newValue);
  });
  const handleBlur = useEventCallback(event => {
    if (clampValue) {
      clamp(event);
    }
  });
  const handleIncrement = useEventCallback(event => {
    applyStep(event, 1);
  });
  const handleDecrement = useEventCallback(event => {
    applyStep(event, -1);
  });
  const handleStepperFocus = useEventCallback(() => {
    inputRef.current.focus();
  });
  const {
    wrapperProps,
    inputProps,
    inputRef
  } = useInput({
    cssModule: "o-ui-number-input",
    id,
    value: !isNil(inputValue) ? inputValue : "",
    placeholder,
    required,
    validationState: !isInRange ? "invalid" : validationState,
    onChange: handleChange,
    type: "number",
    autoFocus,
    disabled,
    readOnly,
    fluid,
    loading,
    active,
    focus,
    hover,
    forwardedRef
  });
  const iconMarkup = useInputIcon(icon, {
    disabled
  });

  const content = /*#__PURE__*/_jsxs(_Fragment, {
    children: [iconMarkup, /*#__PURE__*/_jsx("input", { ...mergeProps(rest, inputProps, {
        onBlur: handleBlur
      })
    }), /*#__PURE__*/_jsx(Spinner, {
      onIncrement: handleIncrement,
      onDecrement: handleDecrement,
      onFocus: handleStepperFocus,
      disabled: readOnly || disabled,
      "aria-hidden": loading
    })]
  });

  return /*#__PURE__*/_jsx(Box, { ...mergeProps(wrapperPropsProp, wrapperProps, {
      className: cssModule("o-ui-input", iconMarkup && "has-icon"),
      as
    }),
    children: content
  });
}
export const NumberInput = forwardRef((props, ref) => /*#__PURE__*/_jsx(InnerNumberInput, { ...props,
  forwardedRef: ref
}));
NumberInput.displayName = "NumberInput";