import { FocusTarget } from "./focusTarget";
import { isFunction, isNil } from "lodash";
import { useMemo } from "react";
export const VirtualFocusCssClass = "o-ui-focus";
export class ElementIterator {
  constructor(elements, {
    from = -1
  } = {}) {
    this.elements = void 0;
    this.index = void 0;
    this.elements = elements;
    this.index = from;
  }

  next() {
    if (this.index < this.elements.length - 1) {
      return this.elements[++this.index];
    }

    return null;
  }

  previous() {
    if (this.index > 0) {
      return this.elements[--this.index];
    }

    return null;
  }

  reset({
    from = -1
  } = {}) {
    this.index = from;
  }

  get currentIndex() {
    return this.index;
  }

}
export class FocusManager {
  constructor(scope, {
    isVirtual = false,
    keyProp
  } = {}) {
    this.scope = void 0;
    this.isVirtual = void 0;
    this.keyProp = void 0;
    this.scope = scope;
    this.isVirtual = isVirtual;
    this.keyProp = keyProp;
  }

  focusElement(element, {
    onFocus,
    onNotFound
  } = {}) {
    if (!isNil(element)) {
      if (this.isVirtual) {
        const {
          elements
        } = this.scope;
        elements.forEach(x => {
          if (x.classList.contains(VirtualFocusCssClass)) {
            x.classList.remove(VirtualFocusCssClass);
          }
        });
        element.classList.add(VirtualFocusCssClass);
      } else {
        if (isFunction(element.focus)) {
          element.focus();

          if (!isNil(onFocus)) {
            onFocus(element);
          }
        }
      }
    } else {
      if (!isNil(onNotFound)) {
        onNotFound();
      }
    }

    return element;
  }

  focusFirst({
    canFocus,
    ...options
  } = {}) {
    const {
      elements
    } = this.scope;
    let target;

    if (elements.length > 0) {
      if (isNil(canFocus)) {
        target = elements[0];
      } else {
        const iterator = new ElementIterator(elements);

        do {
          target = iterator.next();
        } while (!isNil(target) && !canFocus(target));
      }
    }

    return this.focusElement(target, options);
  }

  focusLast({
    canFocus,
    ...options
  } = {}) {
    const {
      elements
    } = this.scope;
    let target;

    if (elements.length > 0) {
      if (isNil(canFocus)) {
        target = elements[elements.length - 1];
      } else {
        const iterator = new ElementIterator(elements, {
          from: elements.length
        });

        do {
          target = iterator.previous();
        } while (!isNil(target) && !canFocus(target));
      }
    }

    return this.focusElement(target, options);
  }

  focusNext({
    canFocus,
    ...options
  } = {}) {
    const {
      elements
    } = this.scope;
    let target;

    if (elements.length > 0) {
      let hasLooped = false;
      canFocus = !isNil(canFocus) ? canFocus : () => true;
      const index = this.isVirtual ? elements.findIndex(x => x.classList.contains(VirtualFocusCssClass)) : elements.indexOf(document.activeElement);
      const iterator = new ElementIterator(elements, {
        from: index !== -1 ? index : undefined
      });

      do {
        target = iterator.next();

        if (isNil(target)) {
          iterator.reset();
        } // If we do a full loop it means there are no focusable elements (probably because of canFocus)
        // therefore we should stop looping to prevent an infinite loop.


        if (iterator.currentIndex === index) {
          hasLooped = true;
        }

        if (!isNil(target) && !canFocus(target)) {
          target = null;
        }
      } while (isNil(target) && !hasLooped);
    }

    return this.focusElement(target, options);
  }

  focusPrevious({
    canFocus,
    ...options
  } = {}) {
    const {
      elements
    } = this.scope;
    let target;

    if (elements.length > 0) {
      let hasLooped = false;
      canFocus = !isNil(canFocus) ? canFocus : () => true;
      const index = this.isVirtual ? elements.findIndex(x => x.classList.contains(VirtualFocusCssClass)) : elements.indexOf(document.activeElement);
      const iterator = new ElementIterator(elements, {
        from: index !== -1 ? index : undefined
      });

      do {
        target = iterator.previous();

        if (isNil(target)) {
          iterator.reset({
            from: elements.length
          });
        } // If we do a full loop it means there are no focusable elements (probably because of canFocus)
        // therefore we should stop looping to prevent an infinite loop.


        if (iterator.currentIndex === index) {
          hasLooped = true;
        }

        if (!isNil(target) && !canFocus(target)) {
          target = null;
        }
      } while (isNil(target) && !hasLooped);
    }

    return this.focusElement(target, options);
  }

  focusKey(key, options) {
    const {
      elements
    } = this.scope;

    if (isNil(this.keyProp)) {
      throw new Error("\"focusKey\" cannot be called without providing a `keyProp` to the FocusManager.");
    }

    return this.focusElement(elements.find(x => x.getAttribute(this.keyProp) === (key === null || key === void 0 ? void 0 : key.toString())), options);
  }

  focusTarget(target, options) {
    switch (target) {
      case FocusTarget.first:
        return this.focusFirst(options);

      case FocusTarget.last:
        return this.focusLast(options);

      default:
        return this.focusKey(target, options);
    }
  }

  search(query, options) {
    const {
      elements
    } = this.scope;
    return this.focusElement(elements.find(x => {
      var _x$textContent;

      return (_x$textContent = x.textContent) === null || _x$textContent === void 0 ? void 0 : _x$textContent.toLowerCase().startsWith(query);
    }), options);
  }

  hasFocus() {
    return !isNil(this.getActiveElement());
  }

  getActiveElement() {
    if (this.isVirtual) {
      const {
        elements
      } = this.scope;
      return elements.find(x => x.classList.contains(VirtualFocusCssClass));
    }

    const activeElement = document.activeElement;

    if (this.scope.isInScope(activeElement)) {
      return activeElement;
    }

    return null;
  }

}
export function useFocusManager(scope, {
  isVirtual,
  keyProp
} = {}) {
  return useMemo(() => new FocusManager(scope, {
    isVirtual,
    keyProp
  }), [scope, isVirtual, keyProp]);
}