import "./Tooltip.css";
import { Children, useCallback, useState } from "react";
import { Overlay, OverlayArrow, isTargetParent, useOverlayLightDismiss, useOverlayPosition, useOverlayTrigger } from "../../overlay";
import { TooltipTriggerContext } from "./TooltipTriggerContext";
import { augmentElement, forwardRef, mergeProps, resolveChildren, useCommittedRef, useControllableState, useEventCallback, useId, useMergedRefs } from "../../shared";
import { isNil } from "lodash";
import { jsx as _jsx } from "react/jsx-runtime";
import { jsxs as _jsxs } from "react/jsx-runtime";
export function parseTooltipTrigger(children) {
  const array = Children.toArray(resolveChildren(children));

  if (array.length !== 2) {
    throw new Error("A tooltip trigger must have exactly 2 children.");
  }

  return array;
}
export function InnerTooltipTrigger({
  open,
  defaultOpen,
  position: positionProp = "top",
  onOpenChange,
  disabled,
  allowFlip = true,
  containerElement,
  zIndex = 10000,
  as = "div",
  children,
  forwardedRef,
  ...rest
}) {
  const [isOpen, setIsOpen] = useControllableState(open, defaultOpen, false);
  const [triggerElement, setTriggerElement] = useState();
  const [overlayElement, setOverlayElement] = useState();
  const [arrowElement, setArrowElement] = useState();
  const overlayRef = useMergedRefs(setOverlayElement, forwardedRef);
  const updateIsOpen = useCallback((event, newValue) => {
    if (isOpen !== newValue) {
      if (!isNil(onOpenChange)) {
        onOpenChange(event, newValue);
      }

      setIsOpen(newValue);
    }
  }, [onOpenChange, isOpen, setIsOpen]);
  const triggerProps = useOverlayTrigger({
    trigger: "hover",
    isOpen,
    onShow: useEventCallback(event => {
      updateIsOpen(event, true);
    }),
    onHide: useEventCallback(event => {
      // Prevent from closing when the focus goes to an element of the overlay on opening.
      if (!isTargetParent(event.relatedTarget, overlayElement)) {
        updateIsOpen(event, false);
      }
    })
  });
  const overlayDismissProps = useOverlayLightDismiss(useCommittedRef(overlayElement), {
    trigger: "hover",
    onHide: useEventCallback(event => {
      // Ignore events related to the trigger.
      if (!isTargetParent(event.target, triggerElement) && event.relatedTarget !== triggerElement) {
        updateIsOpen(event, false);
      }
    }),
    hideOnEscape: true,
    hideOnLeave: true,
    hideOnOutsideClick: false
  });
  const {
    overlayStyles,
    overlayProps: overlayPositionProps,
    arrowStyles
  } = useOverlayPosition(triggerElement, overlayElement, {
    arrowElement,
    position: positionProp,
    allowFlip,
    boundaryElement: containerElement,
    // Not accepting prevent overflow feature because when the tooltip is big enough, it cause arrow render issue sometimes when the position is left or right.
    allowPreventOverflow: false
  });
  const [trigger, tooltip] = parseTooltipTrigger(children);
  const tooltipId = useId(tooltip.props.id, tooltip.props.id ? null : "o-ui-tooltip");
  const triggerMarkup = augmentElement(trigger, mergeProps(!disabled ? triggerProps : {}, {
    "aria-describedby": !disabled && isOpen ? tooltipId : undefined,
    ref: setTriggerElement
  }));
  const tooltipMarkup = augmentElement(tooltip, {
    id: tooltipId
  });
  return /*#__PURE__*/_jsxs(TooltipTriggerContext.Provider, {
    value: {
      isOpen
    },
    children: [triggerMarkup, /*#__PURE__*/_jsxs(Overlay, { ...mergeProps(rest, {
        show: !disabled && isOpen,
        borderOffset: "var(--o-ui-global-scale-charlie)",
        zIndex,
        style: overlayStyles,
        role: "tooltip",
        as,
        ref: overlayRef
      }, overlayDismissProps, overlayPositionProps),
      children: [tooltipMarkup, /*#__PURE__*/_jsx(OverlayArrow, {
        style: arrowStyles,
        ref: setArrowElement
      })]
    })]
  });
}
export const TooltipTrigger = forwardRef((props, ref) => /*#__PURE__*/_jsx(InnerTooltipTrigger, { ...props,
  forwardedRef: ref
}));
TooltipTrigger.displayName = "TooltipTrigger";